@@ -4,19 +4,20 @@ use strict;
 use warnings;
 
 use App::Ack ();
-use App::Ack::ConfigDefault ();
-use App::Ack::ConfigFinder ();
 use App::Ack::Filter;
-use App::Ack::Filter::Collection;
 use App::Ack::Filter::Default;
 use Carp 1.04 ();
-use Getopt::Long 2.38 ();
+use Getopt::Long 2.36 ();
 use Text::ParseWords 3.1 ();
 
+=head1 App::Ack::ConfigLoader
+
 =head1 FUNCTIONS
 
 =head2 process_args( @sources )
 
+
+
 =cut
 
 my @INVALID_COMBINATIONS;
@@ -46,58 +47,6 @@ BEGIN {
     );
 }
 
-sub _generate_ignore_dir {
-    my ( $option_name, $opt ) = @_;
-
-    my $is_inverted = $option_name =~ /^--no/;
-
-    return sub {
-        my ( undef, $dir ) = @_;
-
-        $dir = App::Ack::remove_dir_sep( $dir );
-        if ( $dir !~ /:/ ) {
-            $dir = 'is:' . $dir;
-        }
-
-        my ( $filter_type, $args ) = split /:/, $dir, 2;
-
-        if ( $filter_type eq 'firstlinematch' ) {
-            Carp::croak( qq{Invalid filter specification "$filter_type" for option '$option_name'} );
-        }
-
-        my $filter = App::Ack::Filter->create_filter($filter_type, split(/,/, $args));
-        my $collection;
-
-        my $previous_inversion_matches = $opt->{idirs} && !($is_inverted xor $opt->{idirs}[-1]->is_inverted());
-
-        if ( $previous_inversion_matches ) {
-            $collection = $opt->{idirs}[-1];
-
-            if ( $is_inverted ) {
-                # XXX this relies on invert of an inverted filter
-                #     to return the original
-                $collection = $collection->invert()
-            }
-        }
-        else {
-            $collection = App::Ack::Filter::Collection->new();
-
-            if ( $is_inverted ) {
-                push @{ $opt->{idirs} }, $collection->invert();
-            }
-            else {
-                push @{ $opt->{idirs} }, $collection;
-            }
-        }
-
-        $collection->add($filter);
-
-        if ( $filter_type eq 'is' ) {
-            $collection->add(App::Ack::Filter::IsPath->new($args));
-        }
-    };
-}
-
 sub process_filter_spec {
     my ( $spec ) = @_;
 
@@ -122,29 +71,6 @@ sub process_filter_spec {
     }
 }
 
-
-sub uninvert_filter {
-    my ( $opt, @filters ) = @_;
-
-    return unless defined $opt->{filters} && @filters;
-
-    # Loop through all the registered filters.  If we hit one that
-    # matches this extension and it's inverted, we need to delete it from
-    # the options.
-    for ( my $i = 0; $i < @{ $opt->{filters} }; $i++ ) {
-        my $opt_filter = @{ $opt->{filters} }[$i];
-
-        # XXX Do a real list comparison? This just checks string equivalence.
-        if ( $opt_filter->is_inverted() && "$opt_filter->{filter}" eq "@filters" ) {
-            splice @{ $opt->{filters} }, $i, 1;
-            $i--;
-        }
-    }
-
-    return;
-}
-
-
 sub process_filetypes {
     my ( $opt, $arg_sources ) = @_;
 
@@ -170,9 +96,6 @@ sub process_filetypes {
             if ( not $value ) {
                 @filters = map { $_->invert() } @filters;
             }
-            else {
-                uninvert_filter( $opt, @filters );
-            }
 
             push @{ $opt->{'filters'} }, @filters;
         };
@@ -210,17 +133,15 @@ sub process_filetypes {
         'type-del=s' => $delete_spec,
     );
 
-    foreach my $source (@{$arg_sources}) {
-        my ( $source_name, $args ) = @{$source}{qw/name contents/};
+    for ( my $i = 0; $i < @{$arg_sources}; $i += 2) {
+        my ( $source_name, $args ) = @{$arg_sources}[ $i, $i + 1];
 
         if ( ref($args) ) {
             # $args are modified in place, so no need to munge $arg_sources
-            local @ARGV = @{$args};
-            Getopt::Long::GetOptions(%type_arg_specs);
-            @{$args} = @ARGV;
+            Getopt::Long::GetOptionsFromArray($args, %type_arg_specs);
         }
         else {
-            ( undef, $source->{contents} ) =
+            ( undef, $arg_sources->[$i + 1] ) =
                 Getopt::Long::GetOptionsFromString($args, %type_arg_specs);
         }
     }
@@ -236,32 +157,25 @@ sub process_filetypes {
     return \%additional_specs;
 }
 
-
 sub removed_option {
     my ( $option, $explanation ) = @_;
 
     $explanation ||= '';
     return sub {
-        warn "Option '$option' is not valid in ack 2.\n$explanation";
+        warn "Option '$option' is not valid in ack 2\n$explanation";
         exit 1;
     };
 }
 
-
 sub get_arg_spec {
     my ( $opt, $extra_specs ) = @_;
 
-    my $dash_a_explanation = <<'EOT';
-You don't need -a, ack 1.x users.  This is because ack 2.x has
--k/--known-types which makes it only select files of known types, rather
-than any text file (which is the behavior of ack 1.x).
-
-If you're surprised to see this message because you didn't put -a on the
-command line, you may have options in an .ackrc, or in the ACKRC_OPTIONS
-environment variable.  Try using the --dump flag to help find it.
+    my $dash_a_explanation = <<EOT;
+This is because we now have -k/--known-types which makes it only select files
+of known types, rather than any text file (which is the behavior of ack 1.x).
 EOT
 
-=begin Adding-Options
+=for Adding-Options
 
     *** IF YOU ARE MODIFYING ACK PLEASE READ THIS ***
 
@@ -274,28 +188,16 @@ EOT
     * Your new option is explained when a user invokes ack --man.
       (See the POD at the end of ./ack)
     * Add your option to t/config-loader.t
-    * Add your option to t/Util.pm#get_options
-    * Add your option's description and aliases to dev/generate-completion-scripts.pl
     * Go through the list of options already available, and consider
       whether your new option can be considered mutually exclusive
       with another option.
-
-=end Adding-Options
-
 =cut
-
-    sub _context_value {
-        my $val = shift;
-
-        # Contexts default to 2.
-        return (!defined($val) || ($val < 0)) ? 2 : $val;
-    }
-
     return {
         1                   => sub { $opt->{1} = $opt->{m} = 1 },
-        'A|after-context:-1'  => sub { shift; $opt->{after_context}  = _context_value(shift) },
-        'B|before-context:-1' => sub { shift; $opt->{before_context} = _context_value(shift) },
-        'C|context:-1'        => sub { shift; $opt->{before_context} = $opt->{after_context} = _context_value(shift) },
+        'A|after-context=i' => \$opt->{after_context},
+        'B|before-context=i'
+                            => \$opt->{before_context},
+        'C|context:i'       => sub { shift; my $val = shift; $opt->{before_context} = $opt->{after_context} = ($val || 2) },
         'a'                 => removed_option('-a', $dash_a_explanation),
         'all'               => removed_option('--all', $dash_a_explanation),
         'break!'            => \$opt->{break},
@@ -306,7 +208,7 @@ EOT
         'color-lineno=s'    => \$ENV{ACK_COLOR_LINENO},
         'column!'           => \$opt->{column},
         count               => \$opt->{count},
-        'create-ackrc'      => sub { print "$_\n" for ( '--ignore-ack-defaults', App::Ack::ConfigDefault::options() ); exit; },
+        'create-ackrc'      => sub { App::Ack::create_ackrc(); exit; },
         'env!'              => sub {
             my ( undef, $value ) = @_;
 
@@ -326,18 +228,19 @@ EOT
         'h|no-filename'     => \$opt->{h},
         'H|with-filename'   => \$opt->{H},
         'i|ignore-case'     => \$opt->{i},
-        'ignore-directory|ignore-dir=s' => _generate_ignore_dir('--ignore-dir', $opt),
-        'ignore-file=s'     => sub {
+        'ignore-directory|ignore-dir=s' # XXX Combine this version with the negated version below
+                            => sub {
+                                my ( undef, $dir ) = @_;
+
+                                $dir = App::Ack::remove_dir_sep( $dir );
+                                if ( $dir !~ /^(?:is|match):/ ) {
+                                    $dir = 'is:' . $dir;
+                                }
+                                push @{ $opt->{idirs} }, $dir;
+                               },
+        'ignore-file=s'    => sub {
                                     my ( undef, $file ) = @_;
-
-                                    my ( $filter_type, $args ) = split /:/, $file, 2;
-
-                                    my $filter = App::Ack::Filter->create_filter($filter_type, split(/,/, $args));
-
-                                    if ( !$opt->{ifiles} ) {
-                                        $opt->{ifiles} = App::Ack::Filter::Collection->new();
-                                    }
-                                    $opt->{ifiles}->add($filter);
+                                    push @{ $opt->{ifiles} }, $file;
                                },
         'lines=s'           => sub { shift; my $val = shift; push @{$opt->{lines}}, $val },
         'l|files-with-matches'
@@ -349,12 +252,26 @@ EOT
         'n|no-recurse'      => \$opt->{n},
         o                   => sub { $opt->{output} = '$&' },
         'output=s'          => \$opt->{output},
-        'pager:s'           => sub {
-            my ( undef, $value ) = @_;
+        'pager=s'           => \$opt->{pager},
+        'noignore-directory|noignore-dir=s'
+                            => sub {
+                                my ( undef, $dir ) = @_;
 
-            $opt->{pager} = $value || $ENV{PAGER};
-        },
-        'noignore-directory|noignore-dir=s' => _generate_ignore_dir('--noignore-dir', $opt),
+                                # XXX can you do --noignore-dir=match,...?
+                                $dir = App::Ack::remove_dir_sep( $dir );
+                                if ( $dir !~ /^(?:is|match):/ ) {
+                                    $dir = 'is:' . $dir;
+                                }
+                                if ( $dir !~ /^(?:is|match):/ ) {
+                                    Carp::croak("invalid noignore-directory argument: '$dir'");
+                                }
+
+                                @{ $opt->{idirs} } = grep {
+                                    $_ ne $dir
+                                } @{ $opt->{idirs} };
+
+                                push @{ $opt->{no_ignore_dirs} }, $dir;
+                               },
         'nopager'           => sub { $opt->{pager} = undef },
         'passthru'          => \$opt->{passthru},
         'print0'            => \$opt->{print0},
@@ -395,12 +312,10 @@ EOT
     }; # arg_specs
 }
 
-
 sub process_other {
     my ( $opt, $extra_specs, $arg_sources ) = @_;
 
-    # Start with default options, minus some annoying ones.
-    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version');
+    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version'); # start with default options, minus some annoying ones
     Getopt::Long::Configure(
         'bundling',
         'no_ignore_case',
@@ -409,8 +324,8 @@ sub process_other {
     my $argv_source;
     my $is_help_types_active;
 
-    foreach my $source (@{$arg_sources}) {
-        my ( $source_name, $args ) = @{$source}{qw/name contents/};
+    for ( my $i = 0; $i < @{$arg_sources}; $i += 2 ) {
+        my ( $source_name, $args ) = @{$arg_sources}[ $i, $i + 1 ];
 
         if ( $source_name eq 'ARGV' ) {
             $argv_source = $args;
@@ -418,13 +333,12 @@ sub process_other {
         }
     }
 
-    if ( $argv_source ) { # This *should* always be true, but you never know...
+    if ( $argv_source ) { # this *should* always be true, but you never know...
         my @copy = @{$argv_source};
-        local @ARGV = @copy;
 
         Getopt::Long::Configure('pass_through');
 
-        Getopt::Long::GetOptions(
+        Getopt::Long::GetOptionsFromArray( \@copy,
             'help-types' => \$is_help_types_active,
         );
 
@@ -433,33 +347,16 @@ sub process_other {
 
     my $arg_specs = get_arg_spec($opt, $extra_specs);
 
-    foreach my $source (@{$arg_sources}) {
-        my ( $source_name, $args ) = @{$source}{qw/name contents/};
-
-        my $args_for_source = $arg_specs;
-
-        if ( $source->{project} ) {
-            my $illegal = sub {
-                die "Options --output, --pager and --match are forbidden in project .ackrc files.\n";
-            };
-
-            $args_for_source = {
-                %{$args_for_source},
-                'output=s' => $illegal,
-                'pager:s'  => $illegal,
-                'match=s'  => $illegal,
-            };
-        }
+    for ( my $i = 0; $i < @{$arg_sources}; $i += 2) {
+        my ($source_name, $args) = @{$arg_sources}[$i, $i + 1];
 
         my $ret;
         if ( ref($args) ) {
-            local @ARGV = @{$args};
-            $ret = Getopt::Long::GetOptions( %{$args_for_source} );
-            @{$args} = @ARGV;
+            $ret = Getopt::Long::GetOptionsFromArray( $args, %{$arg_specs} );
         }
         else {
-            ( $ret, $source->{contents} ) =
-                Getopt::Long::GetOptionsFromString( $args, %{$args_for_source} );
+            ( $ret, $arg_sources->[$i + 1] ) =
+                Getopt::Long::GetOptionsFromString( $args, %{$arg_specs} );
         }
         if ( !$ret ) {
             if ( !$is_help_types_active ) {
@@ -477,28 +374,23 @@ sub process_other {
     return;
 }
 
-
 sub should_dump_options {
     my ( $sources ) = @_;
 
-    foreach my $source (@{$sources}) {
-        my ( $name, $options ) = @{$source}{qw/name contents/};
-
+    for(my $i = 0; $i < @{$sources}; $i += 2) {
+        my ( $name, $options ) = @{$sources}[$i, $i + 1];
         if($name eq 'ARGV') {
             my $dump;
-            local @ARGV = @{$options};
             Getopt::Long::Configure('default', 'pass_through', 'no_auto_help', 'no_auto_version');
-            Getopt::Long::GetOptions(
+            Getopt::Long::GetOptionsFromArray($options,
                 'dump' => \$dump,
             );
-            @{$options} = @ARGV;
             return $dump;
         }
     }
     return;
 }
 
-
 sub explode_sources {
     my ( $sources ) = @_;
 
@@ -528,39 +420,34 @@ sub explode_sources {
         delete $arg_spec->{$arg};
     };
 
-    foreach my $source (@{$sources}) {
-        my ( $name, $options ) = @{$source}{qw/name contents/};
+    for(my $i = 0; $i < @{$sources}; $i += 2) {
+        my ( $name, $options ) = @{$sources}[$i, $i + 1];
         if ( ref($options) ne 'ARRAY' ) {
-            $source->{contents} = $options =
+            $sources->[$i + 1] = $options =
                 [ Text::ParseWords::shellwords($options) ];
         }
-
-        for my $j ( 0 .. @{$options}-1 ) {
+        for ( my $j = 0; $j < @{$options}; $j++ ) {
             next unless $options->[$j] =~ /^-/;
             my @chunk = ( $options->[$j] );
             push @chunk, $options->[$j] while ++$j < @{$options} && $options->[$j] !~ /^-/;
             $j--;
 
             my @copy = @chunk;
-            local @ARGV = @chunk;
-            Getopt::Long::GetOptions(
+            Getopt::Long::GetOptionsFromArray(\@chunk,
                 'type-add=s' => $add_type,
                 'type-set=s' => $add_type,
                 'type-del=s' => $del_type,
             );
-            Getopt::Long::GetOptions( %{$arg_spec} );
+            Getopt::Long::GetOptionsFromArray(\@chunk, %{$arg_spec});
 
-            push @new_sources, {
-                name     => $name,
-                contents => \@copy,
-            };
+            splice @copy, -1 * @chunk if @chunk; # XXX explain this
+            push @new_sources, $name, \@copy;
         }
     }
 
     return \@new_sources;
 }
 
-
 sub compare_opts {
     my ( $a, $b ) = @_;
 
@@ -573,7 +460,6 @@ sub compare_opts {
     return $first_a cmp $first_b;
 }
 
-
 sub dump_options {
     my ( $sources ) = @_;
 
@@ -582,8 +468,8 @@ sub dump_options {
     my %opts_by_source;
     my @source_names;
 
-    foreach my $source (@{$sources}) {
-        my ( $name, $contents ) = @{$source}{qw/name contents/};
+    for(my $i = 0; $i < @{$sources}; $i += 2) {
+        my ( $name, $contents ) = @{$sources}[$i, $i + 1];
         if ( not $opts_by_source{$name} ) {
             $opts_by_source{$name} = [];
             push @source_names, $name;
@@ -602,14 +488,13 @@ sub dump_options {
     return;
 }
 
-
 sub remove_default_options_if_needed {
     my ( $sources ) = @_;
 
     my $default_index;
 
-    foreach my $index ( 0 .. $#{$sources} ) {
-        if ( $sources->[$index]{'name'} eq 'Defaults' ) {
+    foreach my $index ( 0 .. $#$sources ) {
+        if ( $sources->[$index] eq 'Defaults' ) {
             $default_index = $index;
             last;
         }
@@ -619,26 +504,25 @@ sub remove_default_options_if_needed {
 
     my $should_remove = 0;
 
-    # Start with default options, minus some annoying ones.
-    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version');
+    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version'); # start with default options, minus some annoying ones
     Getopt::Long::Configure(
         'no_ignore_case',
         'no_auto_abbrev',
         'pass_through',
     );
 
-    foreach my $index ( $default_index + 1 .. $#{$sources} ) {
-        my ( $name, $args ) = @{$sources->[$index]}{qw/name contents/};
+    foreach my $index ( $default_index + 2 .. $#$sources ) {
+        next if $index % 2 != 0;
+
+        my ( $name, $args ) = @{$sources}[ $index, $index + 1 ];
 
         if (ref($args)) {
-            local @ARGV = @{$args};
-            Getopt::Long::GetOptions(
+            Getopt::Long::GetOptionsFromArray($args,
                 'ignore-ack-defaults' => \$should_remove,
             );
-            @{$args} = @ARGV;
         }
         else {
-            ( undef, $sources->[$index]{contents} ) = Getopt::Long::GetOptionsFromString($args,
+            ( undef, $sources->[$index + 1] ) = Getopt::Long::GetOptionsFromString($args,
                 'ignore-ack-defaults' => \$should_remove,
             );
         }
@@ -650,11 +534,10 @@ sub remove_default_options_if_needed {
     return $sources unless $should_remove;
 
     my @copy = @{$sources};
-    splice @copy, $default_index, 1;
+    splice @copy, $default_index, 2;
     return \@copy;
 }
 
-
 sub check_for_mutually_exclusive_options {
     my ( $arg_sources ) = @_;
 
@@ -675,8 +558,7 @@ sub check_for_mutually_exclusive_options {
     while( @copy ) {
         my %set_opts;
 
-        my $source = shift @copy;
-        my ( $source_name, $args ) = @{$source}{qw/name contents/};
+        my ( $source_name, $args ) = splice @copy, 0, 2;
         $args = ref($args) ? [ @{$args} ] : [ Text::ParseWords::shellwords($args) ];
 
         foreach my $opt ( @{$args} ) {
@@ -703,17 +585,12 @@ sub check_for_mutually_exclusive_options {
             }
         }
     }
-
-    return;
 }
 
-
 sub process_args {
     my $arg_sources = \@_;
 
-    my %opt = (
-        pager => $ENV{ACK_PAGER_COLOR} || $ENV{ACK_PAGER},
-    );
+    my %opt;
 
     check_for_mutually_exclusive_options($arg_sources);
 
@@ -727,8 +604,7 @@ sub process_args {
     my $type_specs = process_filetypes(\%opt, $arg_sources);
     process_other(\%opt, $type_specs, $arg_sources);
     while ( @{$arg_sources} ) {
-        my $source = shift @{$arg_sources};
-        my ( $source_name, $args ) = @{$source}{qw/name contents/};
+        my ( $source_name, $args ) = splice( @{$arg_sources}, 0, 2 );
 
         # All of our sources should be transformed into an array ref
         if ( ref($args) ) {
@@ -745,79 +621,11 @@ sub process_args {
     }
     my $filters = ($opt{filters} ||= []);
 
-    # Throw the default filter in if no others are selected.
+    # throw the default filter in if no others are selected
     if ( not grep { !$_->is_inverted() } @{$filters} ) {
         push @{$filters}, App::Ack::Filter::Default->new();
     }
     return \%opt;
 }
 
-
-sub retrieve_arg_sources {
-    my @arg_sources;
-
-    my $noenv;
-    my $ackrc;
-
-    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version');
-    Getopt::Long::Configure('pass_through');
-    Getopt::Long::Configure('no_auto_abbrev');
-
-    Getopt::Long::GetOptions(
-        'noenv'   => \$noenv,
-        'ackrc=s' => \$ackrc,
-    );
-
-    Getopt::Long::Configure('default', 'no_auto_help', 'no_auto_version');
-
-    my @files;
-
-    if ( !$noenv ) {
-        my $finder = App::Ack::ConfigFinder->new;
-        @files  = $finder->find_config_files;
-    }
-    if ( $ackrc ) {
-        # We explicitly use open so we get a nice error message.
-        # XXX This is a potential race condition!.
-        if(open my $fh, '<', $ackrc) {
-            close $fh;
-        }
-        else {
-            die "Unable to load ackrc '$ackrc': $!"
-        }
-        push( @files, { path => $ackrc } );
-    }
-
-    push @arg_sources, {
-        name     => 'Defaults',
-        contents => [ App::Ack::ConfigDefault::options_clean() ],
-    };
-
-    foreach my $file ( @files) {
-        my @lines = App::Ack::ConfigFinder::read_rcfile($file->{path});
-
-        if(@lines) {
-            push @arg_sources, {
-                name     => $file->{path},
-                contents => \@lines,
-                project  => $file->{project},
-            };
-        }
-    }
-
-    if ( $ENV{ACK_OPTIONS} && !$noenv ) {
-        push @arg_sources, {
-            name     => 'ACK_OPTIONS',
-            contents => $ENV{ACK_OPTIONS},
-        };
-    }
-
-    push @arg_sources, {
-        name     => 'ARGV',
-        contents => [ @ARGV ],
-    };
-
-    return @arg_sources;
-}
-
 1; # End of App::Ack::ConfigLoader
