@@ -2,6 +2,8 @@ package App::Ack::Filter;
 
 use strict;
 use warnings;
+use overload
+    '""' => 'to_string';
 
 use App::Ack::Filter::Inverse ();
 use Carp 1.04 ();
@@ -76,8 +78,7 @@ sub register_filter {
 =head2 $filter->filter($resource)
 
 Must be implemented by filter implementations.  Returns
-true if the filter passes, false otherwise.  This method
-must B<not> alter the passed-in C<$resource> object.
+true if the filter passes, false otherwise.
 
 =head2 $filter->invert()
 
